@@ -124,6 +124,11 @@ public class IMAPCapabilities extends MailCapabilities {
      */
     public static final String CAP_SEARCH_FILENAME = "SEARCH=X-MIMEPART";
 
+    /**
+     * TEXT PREVIEW: <code>"SNIPPET=FUZZY"</code>
+     */
+    public static final String CAP_TEXT_PREVIEW = "SNIPPET=FUZZY";
+
     /*-
      * IMAP bit constants
      */
@@ -162,6 +167,8 @@ public class IMAPCapabilities extends MailCapabilities {
     private boolean hasChildren;
     private boolean hasSortDisplay;
     private boolean hasFileNameSearch;
+    private boolean hasAttachmentSearch;
+    private boolean hasTextPreview;
 
     /**
      * Initializes a new {@link IMAPCapabilities}
@@ -305,7 +312,8 @@ public class IMAPCapabilities extends MailCapabilities {
             ", hasThreadReferences=").append(hasThreadReferences()).append(", hasChildren=").append(hasChildren()).append(", hasIMAP4=").append(
             hasIMAP4()).append(", hasIMAP4rev1=").append(hasIMAP4rev1()).append(", hasNamespace=").append(hasNamespace()).append(
             ", hasThreadOrderedSubject=").append(hasThreadOrderedSubject()).append(", hasUIDPlus=").append(hasUIDPlus()).append(
-            ", hasSortDisplay=").append(hasSortDisplay()).append(", hasFileNameSearch=").append(hasFileNameSearch()).toString();
+            ", hasSortDisplay=").append(hasSortDisplay()).append(", hasFileNameSearch=").append(hasFileNameSearch())
+            .append(", hasAttachmentSearch=").append(hasAttachmentMarker()).toString();
     }
 
     /**
@@ -322,6 +330,29 @@ public class IMAPCapabilities extends MailCapabilities {
         return hasFileNameSearch;
     }
 
+    /**
+     * Sets whether text previews are supported
+     *
+     * @param hasTextPreview <code>true</code> to signal support for text previews; otherwise <code>false</code>
+     */
+    public void setTextPreview(boolean hasTextPreview) {
+        this.hasTextPreview = hasTextPreview;
+    }
+
+    @Override
+    public boolean hasTextPreview() {
+        return hasTextPreview;
+    }
+
+    @Override
+    public boolean hasAttachmentMarker() {
+        return hasAttachmentSearch;
+    }
+
+    public void setAttachmentSearchEnabled(boolean enabled) {
+        this.hasAttachmentSearch = enabled;
+    }
+
     @Override
     public boolean hasFolderValidity() {
         return true;
