@@ -213,6 +213,7 @@ public final class CapabilitiesCache {
             imapCaps.setChildren(map.containsKey(IMAPCapabilities.CAP_CHILDREN));
             imapCaps.setHasSubscription(!imapConfig.getIMAPProperties().isIgnoreSubscription());
             imapCaps.setFileNameSearch(map.containsKey(IMAPCapabilities.CAP_SEARCH_FILENAME));
+            imapCaps.setTextPreview(map.containsKey(IMAPCapabilities.CAP_TEXT_PREVIEW));
             if (hasSort && imapConfig.getIMAPProperties().isImapSort()) {
                 // IMAP sort supported & enabled
                 try {
@@ -224,6 +225,7 @@ public final class CapabilitiesCache {
                 // The in-memory sorting does sort with primary respect to display name, the actual address
                 imapCaps.setSortDisplay(true);
             }
+            imapCaps.setAttachmentSearchEnabled(imapConfig.getIMAPProperties().isAttachmentMarkerEnabled());
             /*
              * ACL extension
              */
