@@ -81,6 +81,13 @@ public interface ListLsubEntry {
      */
     boolean exists();
 
+    /**
+     * Indicates whether associated IMAP folder exists and does not hold <code>"\NonExistent"</code> attribute.
+     *
+     * @return <code>true</code> if associated IMAP folder <i>really</i> exists; otherwise <code>false</code>
+     */
+    boolean existsAndIsNotNonExistent();
+
     /**
      * Gets this LIST/LSUB entry's parent or <code>null</code> if no parent exists.
      *
@@ -107,7 +114,9 @@ public interface ListLsubEntry {
      *
      * @return The original full name or <code>null</code>
      */
-    String optOriginalFullName();
+    default String optOriginalFullName() {
+        return null;
+    }
 
     /**
      * Gets the attributes
