@@ -47,7 +47,6 @@
  *
  */
 
-
 package com.openexchange.imap.config;
 
 import java.util.Map;
@@ -210,6 +209,13 @@ public interface IIMAPProperties extends IMailProperties {
      */
     public String getSSLCipherSuites();
 
+    /**
+     * Checks if attachment marker is enabled.
+     *
+     * @return <code>true</code> if attachment marker is enabled for the underlying IMAP; otherwise <code>false</code>
+     */
+    public boolean isAttachmentMarkerEnabled();
+
     /**
      * Gets the map holding IMAP servers with new ACL Extension.
      *
