@@ -66,6 +66,7 @@ import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.ping.IMAPCapabilityAndGreetingCache;
+import com.openexchange.imap.util.HostAndPort;
 import com.sun.mail.imap.ACL;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
@@ -164,7 +165,7 @@ public final class Entity2ACLAutoDetector {
 
         @Override
         public Entity2ACL call() throws Exception {
-            final String greeting = IMAPCapabilityAndGreetingCache.getGreeting(serverUrl, imapConfig.isSecure(), imapConfig.getIMAPProperties());
+            final String greeting = IMAPCapabilityAndGreetingCache.getGreeting(HostAndPort.instanceFor(serverUrl), imapConfig.isSecure(), imapConfig.getIMAPProperties());
             return implFor(greeting, imapConfig);
         }
 
