@@ -56,8 +56,10 @@ import java.util.Comparator;
 import java.util.ConcurrentModificationException;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
+import com.openexchange.java.Strings;
 import com.openexchange.mail.MailSortField;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.utils.MailMessageComparator;
@@ -84,7 +86,9 @@ public final class Conversation {
     private final Set<String> references;
 
     private boolean messageIdsEmpty;
-    private boolean referencesEmpty;
+    private final boolean referencesEmpty;
+
+    private Conversation main;
 
     /**
      * Initializes a new {@link Conversation}.
@@ -92,8 +96,8 @@ public final class Conversation {
     public Conversation() {
         super();
         messages = new HashSet<MailMessageWrapper>(DEFAULT_INITIAL_CAPACITY);
-        messageIds = new HashSet<String>(DEFAULT_INITIAL_CAPACITY << 1, 0.9f);
-        references = new HashSet<String>(DEFAULT_INITIAL_CAPACITY << 1, 0.9f);
+        messageIds = new LinkedHashSet<String>(DEFAULT_INITIAL_CAPACITY << 1, 0.9f);
+        references = new LinkedHashSet<String>(DEFAULT_INITIAL_CAPACITY << 1, 0.9f);
 
         messageIdsEmpty = true;
         referencesEmpty = true;
@@ -132,31 +136,33 @@ public final class Conversation {
         return this;
     }
 
+    /**
+     * Adds the given {@link MailMessageWrapper} to this conversation
+     *
+     * @param mmw The {@link MailMessageWrapper}
+     */
     private void addWrapper(final MailMessageWrapper mmw) {
-        if (messages.add(mmw)) {
+        Conversation main = getMain();
+        if (main.messages.add(mmw)) {
             final MailMessage message = mmw.message;
             final String messageId = message.getMessageId();
             if (null != messageId) {
-                if (messageIds.add(messageId)) {
-                    messageIdsEmpty = false;
+                if (main.messageIds.add(messageId)) {
+                    main.messageIdsEmpty = false;
                 }
             }
 
             final String[] sReferences = message.getReferences();
             if (null != sReferences) {
                 for (final String sReference : sReferences) {
-                    if (null != sReference) {
-                        if (references.add(sReference)) {
-                            referencesEmpty = false;
-                        }
+                    if (null != sReference && Strings.isNotEmpty(sReference)) {
+                        main.references.add(sReference);
                     }
                 }
             } else {
                 String inReplyTo = message.getInReplyTo();
-                if (null != inReplyTo) {
-                    if (references.add(inReplyTo)) {
-                        referencesEmpty = false;
-                    }
+                if (null != inReplyTo && Strings.isNotEmpty(inReplyTo)) {
+                    main.references.add(inReplyTo);
                 }
             }
         }
@@ -166,18 +172,62 @@ public final class Conversation {
      * Joins this conversation with other conversation.
      *
      * @param other The other conversation to join with
-     * @return This conversation
+     * @return The main conversation
      */
     public Conversation join(final Conversation other) {
         if (null != other) {
-            final Set<MailMessageWrapper> messages = other.messages;
+            if (this.equals(other)) {
+                return this;
+            }
+            Conversation main = this.main;
+            if (null != main) {
+                main.join(other);
+                return main.getMain();
+            }
+
+            // now get the other main
+            Conversation otherMain = other.getMain();
+            // special case, we found ourself, no need to fetch mails
+            if (this.equals(otherMain)) {
+                return this;
+            }
+
+            final Set<MailMessageWrapper> messages = otherMain.messages;
             for (final MailMessageWrapper mmw : messages) {
                 addWrapper(mmw);
             }
+            otherMain.setMain(this);
         }
         return this;
     }
 
+    /**
+     * Sets the main {@link Conversation}
+     *
+     * @param main The {@link Conversation} to set as main
+     */
+    private void setMain(Conversation main) {
+        if (this.equals(main)) {
+            return;
+        }
+        Conversation otherMain = getMain();
+        if (!this.equals(otherMain)) {
+            otherMain.setMain(main);
+        }
+        this.main = main;
+    }
+
+    /**
+     * Gets the main {@link Conversation}
+     *
+     * @return The main {@link Conversation}
+     */
+    private Conversation getMain() {
+        Conversation main = this.main;
+        return main != null ? main.getMain() : this;
+    }
+
+
     /**
      * Checks if this conversation references OR is referenced by given message
      *
@@ -295,14 +345,23 @@ public final class Conversation {
         return false;
     }
 
+    /**
+     * Returns true if this is the main {@link Conversation}
+     * @return <code>true</code> if this is the main  {@link Conversation}, <code>false</code> otherwise
+     */
+    public boolean isMain() {
+        return this.equals(getMain());
+    }
+
     /**
      *Adds the message identifiers from this conversation to given set.
      *
      * @param set The set to add the message identifiers to
      */
     public void addMessageIdsTo(Set<String> set) {
-        if (!messageIdsEmpty) {
-            set.addAll(messageIds);
+        Conversation main = getMain();
+        if (!main.messageIdsEmpty) {
+            set.addAll(main.messageIds);
         }
     }
 
@@ -322,17 +381,36 @@ public final class Conversation {
      * @return The messages with given sorting
      */
     public List<MailMessage> getMessages(final Comparator<MailMessage> comparator) {
-        if (messages.isEmpty()) {
+        Conversation main = getMain();
+        if (main.messages.isEmpty()) {
             return Collections.emptyList();
         }
-        final List<MailMessage> ret = new ArrayList<MailMessage>(messages.size());
-        for (final MailMessageWrapper mmw : messages) {
+        final List<MailMessage> ret = new ArrayList<MailMessage>(main.messages.size());
+        for (final MailMessageWrapper mmw : main.messages) {
             ret.add(mmw.message);
         }
         Collections.sort(ret, null == comparator ? COMPARATOR_DESC : comparator);
         return ret;
     }
 
+    /**
+     * Gets the references
+     *
+     * @return The references
+     */
+    public Set<String> getReferences() {
+        return getMain().references;
+    }
+
+    /**
+     * Gets the message ids
+     *
+     * @return The message ids
+     */
+    public Set<String> getMessageIds() {
+        return getMain().messageIds;
+    }
+
     /**
      * Simple wrapper class for having a message as hash key.
      * <p>
@@ -343,6 +421,11 @@ public final class Conversation {
         final MailMessage message;
         private final int hash;
 
+        /**
+         * Initializes a new {@link MailMessageWrapper}.
+         *
+         * @param message The {@link MailMessage} to wrap
+         */
         MailMessageWrapper(final MailMessage message) {
             super();
             this.message = message;
@@ -387,4 +470,15 @@ public final class Conversation {
         }
     } // End of class MailMessageWrapper
 
+    @Override
+    public String toString() {
+        StringBuilder builder = new StringBuilder();
+        builder.append("Conversation [messageIds=");
+        builder.append(messageIds);
+        builder.append(", references=");
+        builder.append(references);
+        builder.append("]");
+        return builder.toString();
+    }
+
 }
