@@ -55,8 +55,10 @@ import static com.openexchange.imap.util.ImapUtility.prepareImapCommandForLoggin
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import javax.mail.FetchProfile;
 import javax.mail.FetchProfile.Item;
 import javax.mail.MessagingException;
@@ -272,7 +274,7 @@ public final class Conversations {
                 final Response response = r[len];
                 if (response.isOK()) {
                     try {
-                        final List<MailMessage> mails = new ArrayList<MailMessage>(messageCount);
+                        final List<MailMessage> mails = new ArrayList<MailMessage>(len);
                         final String fullName = imapFolder.getFullName();
                         final String sInReplyTo = "In-Reply-To";
                         final String sReferences = "References";
@@ -357,23 +359,110 @@ public final class Conversations {
      * @return The folded conversations
      */
     public static List<Conversation> fold(final List<Conversation> toFold) {
-        Iterator<Conversation> iter = toFold.iterator();
-        int i = 0;
-        while (iter.hasNext()) {
-            foldInto(iter.next(), iter);
-            iter = toFold.listIterator(++i);
-        }
+        fold(toFold, null);
         return toFold;
     }
 
-    private static void foldInto(final Conversation conversation, final Iterator<Conversation> iter) {
-        while (iter.hasNext()) {
-            final Conversation other = iter.next();
-            if (conversation.referencesOrIsReferencedBy(other)) {
-                iter.remove();
-                conversation.join(other);
+    private static void fold(final List<Conversation> toFold, Map<String, Conversation> lookupTable) {
+        // first collect a temporal lookup table and fold for the first time
+        {
+            Map<String, Conversation> tempLookupTable = new HashMap<String, Conversation>(toFold.size());
+            for (Iterator<Conversation> iter = toFold.iterator(); iter.hasNext();) {
+                Conversation conversation = iter.next();
+                
+                boolean removed = false;
+                for (String messageId : conversation.getMessageIds()) {
+                    Conversation existing = tempLookupTable.putIfAbsent(messageId, conversation);
+                    if (null != existing && !conversation.equals(existing)) {
+                        if (!removed) {
+                            iter.remove();
+                            removed = true;
+                        }
+                        Conversation joined = existing.join(conversation);
+                        if (!joined.equals(existing)) {
+                            tempLookupTable.put(messageId, joined);
+                        }
+                    }
+                }
+                
+                for (String reference : conversation.getReferences()) {
+                    Conversation existing = tempLookupTable.putIfAbsent(reference, conversation);
+                    if (null != existing && !conversation.equals(existing)) {
+                        if (!removed) {
+                            iter.remove();
+                            removed = true;
+                        }
+                        Conversation joined = existing.join(conversation);
+                        if (!joined.equals(existing)) {
+                            tempLookupTable.put(reference, joined);
+                        }
+                    }
+                }
+            }
+        }
+        // iterate a second time and remove all that are not main Conversations
+        {
+            for (Iterator<Conversation> iter = toFold.iterator(); iter.hasNext();) {
+                Conversation conversation = iter.next();
+                if (!conversation.isMain()) {
+                    iter.remove();
+                }
+            }
+        }
+        // now fill in the results if lookupTable is present
+        if (null == lookupTable) {
+            return;
+        }
+        {
+            for (Iterator<Conversation> iter = toFold.iterator(); iter.hasNext();) {
+                Conversation conversation = iter.next();
+                for (String messageId : conversation.getMessageIds()) {
+                    Conversation existing = lookupTable.putIfAbsent(messageId, conversation);
+                    if (null != existing && !conversation.equals(existing)) {
+                        LOG.debug("folding of Conversations failed for {} and {}", existing, conversation);
+                    }
+                }
+                for (String reference : conversation.getReferences()) {
+                    Conversation existing = lookupTable.putIfAbsent(reference, conversation);
+                    if (null != existing && !conversation.equals(existing)) {
+                        LOG.debug("folding of Conversations failed for {} and {}", existing, conversation);
+                    }
+                }
             }
         }
     }
 
+    /**
+     * Folds specified conversations and adds MaillMessages to the list.
+     *
+     * @param toFold The conversations to fold
+     * @param toMergeWith The List of MaillMessages to be added
+     * @return The folded conversations
+     */
+    public static List<Conversation> foldAndMergeWithList(final List<Conversation> toFold, List<MailMessage> toMergeWith) {
+        HashMap<String, Conversation> lookupTable = new HashMap<String, Conversation>(toFold.size() * 2);
+        fold(toFold, lookupTable);
+        for (MailMessage mailMessage : toMergeWith) {
+            String messageId = mailMessage.getMessageId();
+            if (null != messageId) {
+                Conversation conversation = lookupTable.get(messageId);
+                if (null != conversation) {
+                    conversation.addMessage(mailMessage);
+                }
+            }
+
+            String[] references = mailMessage.getReferences();
+            if (null != references) {
+                for (String reference : references) {
+                    Conversation conversation = lookupTable.get(reference);
+                    if (null != conversation) {
+                        conversation.addMessage(mailMessage);
+                    }
+                }
+            }
+
+        }
+        return toFold;
+    }
+
 }
